//	VirtualDub - Video processing and capture application
//	Copyright (C) 1998-2001 Avery Lee
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "stdafx.h"

#include <windows.h>
#include <vfw.h>

#include "resource.h"
#include "auxdlg.h"
#include "oshelper.h"

#include <vd2/Priss/decoder.h>
#include <vd2/system/thread.h>
#include <vd2/system/profile.h>
#include "LogWindow.h"
#include "RTProfileDisplay.h"
// ********************************************************************
// *** VirtualDubMod												***
// *** Cyrius, 11/11/2002											***
// BEGIN **************************************************************
#include "gui.h"
#include "misc.h"
// END ****************************************************************

extern "C" unsigned long version_num;
extern "C" char version_time[];

extern HINSTANCE g_hInst;

static volatile HWND g_hwndLogWindow = NULL;
static volatile HWND g_hwndProfileWindow = NULL;

class VDLogWindowThread : public VDThread {
public:
	VDSignal mReady;

	VDLogWindowThread() : VDThread("Log Window") {}

	void ThreadRun() {
		mReady.signal();
		// There is a race condition here that can allow two log windows to appear,
		// but that is not a big deal.
		if (!g_hwndLogWindow) {
			DialogBox(g_hInst, MAKEINTRESOURCE(IDD_LOG), NULL, LogDlgProc);
			g_hwndLogWindow = 0;
		} else
			SetWindowPos(g_hwndLogWindow, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
	}

	static BOOL CALLBACK LogDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
		switch(msg) {
		case WM_INITDIALOG:
			VDGetILogWindowControl(GetDlgItem(hdlg, IDC_LOG))->AttachAsLogger(false);
			g_hwndLogWindow = hdlg;
		case WM_SIZE:
			{
				RECT r;

				GetClientRect(hdlg, &r);
				SetWindowPos(GetDlgItem(hdlg, IDC_LOG), NULL, 0, 0, r.right, r.bottom, SWP_NOZORDER|SWP_NOACTIVATE);
			}
			return TRUE;
		case WM_COMMAND:
			if (LOWORD(wParam) == IDCANCEL)
				EndDialog(hdlg, 0);
			return TRUE;
		}
		return FALSE;
	}
};

extern void VDOpenLogWindow() {
	VDLogWindowThread logwin;

	logwin.ThreadStart();
	if (logwin.isThreadAttached()) {
		logwin.mReady.wait();
		logwin.ThreadDetach();
	}
}


class VDProfileWindowThread : public VDThread {
public:
	VDSignal mReady;

	VDProfileWindowThread() : VDThread("Profile Window") {}

	void ThreadRun() {
		mReady.signal();
		// There is a race condition here that can allow two Profile windows to appear,
		// but that is not a big deal.
		if (!g_hwndProfileWindow) {
			VDInitProfilingSystem();
			DialogBox(g_hInst, MAKEINTRESOURCE(IDD_PROFILER), NULL, ProfileDlgProc);
			g_hwndProfileWindow = 0;
		} else
			SetWindowPos(g_hwndProfileWindow, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
	}

	static BOOL CALLBACK ProfileDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
		switch(msg) {
		case WM_INITDIALOG:
			g_hwndProfileWindow = hdlg;
			VDGetIRTProfileDisplayControl(GetDlgItem(hdlg, IDC_PROFILE))->SetProfiler(VDGetRTProfiler());
		case WM_SIZE:
			{
				RECT r;

				GetClientRect(hdlg, &r);
				SetWindowPos(GetDlgItem(hdlg, IDC_PROFILE), NULL, 0, 0, r.right, r.bottom, SWP_NOZORDER|SWP_NOACTIVATE);
			}
			return TRUE;
		case WM_COMMAND:
			if (LOWORD(wParam) == IDCANCEL)
				EndDialog(hdlg, 0);
			return TRUE;
		}
		return FALSE;
	}
};

extern void VDOpenProfileWindow() {
	VDProfileWindowThread profwin;

	profwin.ThreadStart();
	if (profwin.isThreadAttached()) {
		profwin.mReady.wait();
		profwin.ThreadDetach();
	}
}


// ********************************************************************
// *** VirtualDubMod												***
// *** Cyrius, 11/11/2002											***
// BEGIN **************************************************************
struct ReposItem ShowTextCtlPosData[]={
	{ IDC_CHANGES		, REPOS_SIZERIGHT | REPOS_SIZEDOWN },
	{ IDOK				, REPOS_MOVERIGHT | REPOS_MOVEDOWN },
	{ 0 }
};

POINT ShowTextCtlPos[2];

BOOL APIENTRY ShowTextDlgProc( HWND hDlg, UINT message, UINT wParam, LONG lParam) {
	static RECT rInitial;
	HRSRC hRSRC;

    switch (message)
    {
		case WM_SIZE:
			guiReposResize(hDlg, ShowTextCtlPosData, ShowTextCtlPos);
			return TRUE;

		case WM_GETMINMAXINFO:
			{
				LPMINMAXINFO lpmmi = (LPMINMAXINFO)lParam;

				lpmmi->ptMinTrackSize.x = rInitial.right - rInitial.left;
				lpmmi->ptMinTrackSize.y = rInitial.bottom - rInitial.top;
			}
			return TRUE;

        case WM_INITDIALOG:
			GetWindowRect(hDlg, &rInitial);
// END ****************************************************************
			if (hRSRC = FindResource(NULL, (LPSTR)lParam, "STUFF")) {
				HGLOBAL hGlobal;
				if (hGlobal = LoadResource(NULL, hRSRC)) {
					LPVOID lpData;

					if (lpData = LockResource(hGlobal)) {
						char *s = (char *)lpData;
						char *ttl = new char[256];

						while(*s!='\r') ++s;
						if (ttl) {
							memcpy(ttl, (char *)lpData, s - (char *)lpData);
							ttl[s-(char*)lpData]=0;
							SendMessage(hDlg, WM_SETTEXT, 0, (LPARAM)ttl);
							delete ttl;
						}
						s+=2;
						SendMessage(GetDlgItem(hDlg, IDC_CHANGES), WM_SETFONT, (WPARAM)GetStockObject(ANSI_FIXED_FONT), MAKELPARAM(TRUE, 0));
						SendMessage(GetDlgItem(hDlg, IDC_CHANGES), WM_SETTEXT, 0, (LPARAM)s);
// ********************************************************************
// *** Cyrius, 11/11/2002											***
// BEGIN **************************************************************
						SendMessage(hDlg, WM_SETICON, (WPARAM)ICON_SMALL, (LPARAM)LoadIcon(g_hInst, MAKEINTRESOURCE(IDI_VIRTUALDUB)));
						guiReposInit(hDlg, ShowTextCtlPosData, ShowTextCtlPos);
// END ****************************************************************
						FreeResource(hGlobal);
						return TRUE;
					}
					FreeResource(hGlobal);
				}
			}
            return FALSE;

        case WM_COMMAND:
            if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
            {
                EndDialog(hDlg, TRUE);  
                return TRUE;
            }
            break;
    }
    return FALSE;
}

BOOL APIENTRY WelcomeDlgProc( HWND hDlg, UINT message, UINT wParam, LONG lParam)
{
    switch (message)
    {
        case WM_COMMAND:
			switch(LOWORD(wParam)) {
			case IDOK: case IDCANCEL:
                EndDialog(hDlg, TRUE);  
                return TRUE;
			case IDC_HELP2:
				VDShowHelp(hDlg);
				return TRUE;
            }
            break;
    }
    return FALSE;
}

void Welcome() {
	DWORD dwSeenIt;

	if (!QueryConfigDword(NULL, "SeenWelcome", &dwSeenIt) || !dwSeenIt) {
		DialogBox(g_hInst, MAKEINTRESOURCE(IDD_WELCOME), NULL, WelcomeDlgProc);

		SetConfigDword(NULL, "SeenWelcome", 1);
	}
}

BOOL APIENTRY AnnounceExperimentalDlgProc( HWND hDlg, UINT message, UINT wParam, LONG lParam)
{
    switch (message)
    {
        case WM_COMMAND:
			switch(LOWORD(wParam)) {
			case IDOK: case IDCANCEL:
                EndDialog(hDlg, TRUE);  
                return TRUE;
			case IDC_VERIFY:
				EnableWindow(GetDlgItem(hDlg, IDOK), IsDlgButtonChecked(hDlg, IDC_VERIFY));
				return TRUE;
            }
            break;
    }
    return FALSE;
}

void AnnounceExperimental() {
#if 0		// 1.5.6 is not experimental
	DWORD dwSeenIt;

	if (!QueryConfigDword(NULL, "SeenExperimental 1.5.5", &dwSeenIt) || !dwSeenIt) {
		DialogBox(g_hInst, MAKEINTRESOURCE(IDD_EXPERIMENTAL), NULL, AnnounceExperimentalDlgProc);

		SetConfigDword(NULL, "SeenExperimental 1.5.5", 1);
	}
#endif
}

// ********************************************************************
// *** VirtualDubMod												***
// *** Cyrius, August 2005											***
// BEGIN **************************************************************
void AnnounceDeprecatedMatroskaSupport() {
	DWORD dwSeenIt;

   if(!QueryConfigDword(NULL, "SeenMatroskaDeprecated 1.5.10.2", &dwSeenIt) || !dwSeenIt) {
		VDStringW message(L"La prise en charge Matroska de VirtualDubMod est dsormais plutt ancienne et basique.\n"
			L"De nombreuses fonctionnalits offertes par ce format ne sont pas prises en charge par ce programme:\n"
			L" - Frquence d'Images Variable\n"
			L" - chapitrage avanc\n"
			L" - marquage avanc\n"
			L" - sous-titres avances tels que SSA, ASS ou VobSub\n"
			L" - priise en charge AVC native\n"
			L" - ...\n"
			L"Afin de pouvoir profiter pleinement de toutes ces fonctionnalits, vous devriez envisager d'utiliser des outils plus rcents tels que AVI-Mux GUI (de Alexander No) ou MKVToolnix (de Moritz Bunkus).\n"
			L"\n"
			L"Pour en savoir plus, veuillez visiter http://www.matroska.org.");
		if(IsWindowsNT())
			MessageBoxW(NULL, message.c_str(), L"Avertissement - Prise en charge Matroska",  MB_OK | MB_ICONWARNING);
		else
			MessageBoxA(NULL, VDTextWToA(message.c_str()).c_str(), "Avertissement - Prise en charge Matroska",  MB_OK | MB_ICONWARNING);
		SetConfigDword(NULL, "SeenMatroskaDeprecated 1.5.10.2", 1);
	}
}
// END ****************************************************************

static const char g_szDivXWarning[]=
	"Avertissement VirtualDub: codec \"DivX\" dtect\0"
	"Un ou plusieurs pilotes \"DivX 3\" ont t dtects sur votre systme. Ces pilotes sont des versions pirates et donc illgales "
	"de pilotes suivants:\r\n"
	"\r\n"
	"* DivX 3 low motion/fast motion: Microsoft MPEG-4 V3 video\r\n"
	"* DivX audio: Microsoft Windows Media Audio\r\n"
	"* \"Radium\" MP3: Fraunhofer-IIS MPEG layer III audio\r\n"
	"\r\n"
	"Ces pilotes sont connus pour tre problmatiques, et peuvent tre  l'origine de soucis de stabilit et d'interfrences "
	"avec les pilotes d'origine. La stabilit de VirtualDub ne peut pas tre garantie avec "
	"ces pilotes. Merci de ne PAS signaler  l'auteur les plantages lis  ces pilotes "
	"puisque ces problmes ne peuvent pas tre corrigs dans VirtualDub mme.\r\n"
	"\r\n"
	"Il ne s'agit que d'un avertissement. Mis  part les avertissements et les corrections de bugs, VirtualDub n'effectue aucune "
	"action supplmentaire au chargement du codec DivX.\r\n"
	"\r\n"
	"REMARQUE: Cet avertissement ne s'applique PAS au DivX 4.0 et aux versions suivantes, qui sont des codecs "
	"compltement diffrents."
	;

static const char g_szAPWarning[]=
	"Avertissement VirtualDub: codec \"AngelPotion Definitive\" dtect\0"
	"Le codec \"AngelPotion Definitive\" a t dtect sur votre systme. Ce pilote est une version pirate et donc illgale "
	"du pilote suivant:\r\n"
	"\r\n"
	"* Microsoft MPEG-4 V3 video\r\n"
	"\r\n"
	"Le codec AngelPotion est un hack particulirement mauvais du codec MS MPEG-4 V3 et est connu pour causer "
	"de graves conflits, notamment (mais pas uniquement):\r\n"
	"* Utilisation excessive du disque temporaire\r\n"
	"* Ractions incorrectes aux formats compresss des autres codecs, et mme au RGB non-compress\r\n"
	"* Empche compltement certaines applications de charger des fichiers AVI\r\n"
	"* Dsactivation du tlchargement automatique de codec de Windows Media Player\r\n"
	"\r\n"
	"L'auteur ne peut absolument pas garantir la stabilit de VirtualDub quand AngelPotion est charg, "
	"mme si le codec n'est pas en cours d'utilisation. Tous les dumps de plantages indiquant que AP est charg seront immdiatement ignors. "
	"Il est recommand de dsinstaller AngelPotion si c'est possible.\r\n"
	"\r\n"
	"Il ne s'agit que d'un avertissement. Mis  part les avertissements et les corrections de bugs, VirtualDub n'effectue aucune "
	"action supplmentaire au chargement du codec AngelPotion."
	;

BOOL APIENTRY DivXWarningDlgProc( HWND hdlg, UINT message, UINT wParam, LONG lParam)
{
	const char *s;

    switch (message)
    {
		case WM_INITDIALOG:
			s = (const char *)lParam;
			SetWindowText(hdlg, s);
			while(*s++);
			SendDlgItemMessage(hdlg, IDC_WARNING, WM_SETTEXT, 0, (LPARAM)s);
			return TRUE;

        case WM_COMMAND:
			switch(LOWORD(wParam)) {
			case IDOK: case IDCANCEL:
                EndDialog(hdlg, TRUE);
                return TRUE;
            }
            break;
    }
    return FALSE;
}

static bool DetectDriver(const char *pszName) {
	char szDriverANSI[256];
	ICINFO info;

	for(int i=0; ICInfo(ICTYPE_VIDEO, i, &info); ++i) {
		if (WideCharToMultiByte(CP_ACP, 0, info.szDriver, -1, szDriverANSI, sizeof szDriverANSI, NULL, NULL)
			&& !stricmp(szDriverANSI, pszName))

			return true;
	}

	return false;
}

void DetectDivX() {
	DWORD dwSeenIt;

	if (!QueryConfigDword(NULL, "SeenDivXWarning", &dwSeenIt) || !dwSeenIt) {
		if (DetectDriver("divxc32.dll") || DetectDriver("divxc32f.dll")) {
			DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDD_DIVX_WARNING), NULL, DivXWarningDlgProc, (LPARAM)g_szDivXWarning);

			SetConfigDword(NULL, "SeenDivXWarning", 1);
		}
	}
	if (!QueryConfigDword(NULL, "SeenAngelPotionWarning", &dwSeenIt) || !dwSeenIt) {
		if (DetectDriver("APmpg4v1.dll")) {

			DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDD_DIVX_WARNING), NULL, DivXWarningDlgProc, (LPARAM)g_szAPWarning);

			SetConfigDword(NULL, "SeenAngelPotionWarning", 1);
		}
	}
}
